## Eingabe: Modell (formula) und Dataframe, keine Einser-Spalte!!
## 
## Grenzen trim
## m: Gridlnge
## h: Bandbreite fr (5.3) in Escanciano-Goh
## bsrep: Anzahl Bootstrap-Wiederholungen
## Ausgabe: Teststatistik und Matrix mit b Bootstrap-Statistiken fr CvM und KS

library(quantreg)
library(np)


eg.lqr.test <- function(formula, dat, bsrep=19, trim=c(.05,.95),m=30,h=30^-.2){
outname <- all.vars(formula)[1]
covname <- all.vars(formula)[-1]
y  <- dat[,outname]
n <- length(y)

gridvals <- seq(trim[1],trim[2],length=m)
summands <- matrix(NA,ncol=bsrep+1,nrow=m)

bernoulli <- matrix(sample(c(.5*(1-sqrt(5)) ,.5*(1+sqrt(5))),n*bsrep,replace=T,prob=c((1+sqrt(5))/(2*sqrt(5)),1-(1+sqrt(5))/(2*sqrt(5)) )),ncol=n,nrow=bsrep)

mod <- rq(formula, data=dat , gridvals)
xbeta <- predict(mod)
X  <- model.matrix(formula, data=dat)[,-1]
k <- length(X[1,])

## fr P_j: Matrix G
  G <- matrix(0,ncol=n,nrow=n)
  Y <- t(X)
  for (i in 1:n) {
    G[,i] <- apply(((Y- X[i,])<=0), 2,prod)
    }

# Berechne psi.j und P.j

for(j in 1:m) { 
 
  ## P_j berechnen (fr P_j: Matrix H_j; fr H_j: delta_nj)

    delta.j <- matrix(0,ncol=k+1,nrow=n)
    for(r in 1:n) {   
    delta.j[r,] <-  npudens(tdat=xbeta[r,], bws=h, edat=xbeta[r,j])$dens* c(1,as.numeric(X[r,]))
      }
    xx <- qr(delta.j)
    P.j <- qr.resid(xx,G)

    #H.j <- (diag(n) - delta.j%*%solve(t(delta.j)%*%delta.j)%*%t(delta.j))
    #P.j <- H.j%*% G
    #rm(H.j,delta.j)

  ## psi_j berechnen
  
   psi.j <- I( y - predict(mod)[,j]  < 0) - gridvals[j]
 
  ## Summanden berechnen; fr Teststatistik und Bootstrap

   summands[j,1] <- sum((t(psi.j) %*% P.j)^2)
      
   for (b in 1:bsrep) {
    summands[j,b+1] <- sum((t(bernoulli[b,]*psi.j) %*% P.j)^2)
   }

   rm(psi.j,P.j)

}


teststat.cvm <- mean(summands[,1])/n^2
teststat.ks <-  0#max(sqrt(summands[,1]))/n^2

teststat.cvm.bs <- apply(summands[,-1], 2, function(x){mean(x)/n^2})
teststat.ks.bs <- 0#apply(summands[,-1], 2, function(x){max(sqrt(x))/n^2})

pval.ks <- 99 #mean(teststat.ks.bs > teststat.ks, na.rm=TRUE)
pval.cvm <- mean(teststat.cvm.bs > teststat.cvm, na.rm=TRUE)


list(teststat.ks=teststat.ks, pval.ks=pval.ks,teststat.cvm=teststat.cvm, pval.cvm=pval.cvm, test="EG Linear Quantile Regression")


}

